@@ -100,11 +100,13 @@
 #' \item{log.like}{The log-likelihood(s), given by the normalising constant(s), computed with the aid of \code{\link[matrixStats]{rowLogSumExps}}.}
 #' }
 #' @seealso \code{\link{mcmc_IMIFA}}, \code{\link[matrixStats]{rowLogSumExps}}
-#' @references Murphy, K., Gormley, I. C. and Viroli, C. (2017) Infinite Mixtures of Infinite Factor Analysers: Nonparametric Model-Based Clustering via Latent Gaussian Models, \code{https://arxiv.org/abs/1701.07010}.
+#' @references Murphy, K., Gormley, I. C. and Viroli, C. (2017) Infinite Mixtures of Infinite Factor Analysers: Nonparametric Model-Based Clustering via Latent Gaussian Models, \href{https://arxiv.org/abs/1701.07010}{arXiv:1701.07010}.
 #'
 #' Yellot, J. I. Jr. (1977) The relationship between Luce's choice axiom, Thurstone's theory of comparative judgment, and the double exponential distribution, \emph{Journal of Mathematical Psychology}, 15: 109-144.
 #' @export
 #'
+#' @author Keefe Murphy
+#'
 #' @examples
 #' # Set the dimensions & simulate a matrix of weights
 #'   N         <- 1
@@ -237,14 +239,14 @@
 #'
 #' @return Either a single rate hyperparameter or \code{ncol(covar)} variable specific hyperparameters.
 #' @export
-#' @importFrom matrixcalc "is.positive.semi.definite"
-#' @importFrom MASS "ginv"
 #'
 #' @seealso \code{\link{mcmc_IMIFA}}
 #' @references Fruwirth-Schnatter, S. and Lopes, H. F. (2010). Parsimonious Bayesian factor analysis when the number of factors is unknown, \emph{Technical Report}. The University of Chicago Booth School of Business.
 #'
 #' Tipping, M. E. and Bishop, C. M. (1999). Probabilistic principal component analysis, \emph{Journal of the Royal Statistical Society: Series B (Statistical Methodology)}, 61(3): 611-622.
 #'
+#' @author Keefe Murphy
+#'
 #' @examples
 #' data(olive)
 #' olive2 <- olive[,-(1:2)]
@@ -255,14 +257,14 @@
 #' rate   <- psi_hyper(shape=3, covar=cov(olive_scaled), type="unconstrained")
 #' rate
     psi_hyper   <- function(shape, covar, type=c("unconstrained", "isotropic")) {
-      if(!all(is.positive.semi.definite(covar),
+      if(!all(is.posi_def(covar, semi=TRUE),
               is.symmetric(covar),
               is.double(covar)))           stop("Invalid covariance matrix supplied")
       if(any(!is.numeric(shape),
              length(shape) != 1))          stop("'shape' must be a single digit")
       inv.cov   <- try(base::solve(covar), silent=TRUE)
       if(inherits(inv.cov, "try-error"))  {
-        inv.cov <- ginv(covar)
+        inv.cov <- .moore_inv(covar)
       }
         unname((shape - 1)/switch(match.arg(type), unconstrained=diag(inv.cov),
                                   isotropic=rep(exp(mean(log(diag(inv.cov)))), ncol(covar))))
@@ -302,6 +304,8 @@
 #'
 #' Durante, D. (2017). A note on the multiplicative gamma process, \emph{Statistics & Probability Letters}, 122: 198-204.
 #'
+#' @author Keefe Murphy
+#'
 #' @examples
 #' # Check if expected shrinkage under the MGP increases with the column index (WRONG!).
 #' MGP_check(ad1=1.5, ad2=1.8, Q=10, nu=2, inverse=FALSE)[[1]]$valid
@@ -357,6 +361,8 @@
 #' @references McNicholas, P. D. and Murphy, T. B. (2008) Parsimonious Gaussian Mixture Models, \emph{Statistics and Computing}, 18(3): 285-296.
 #' @seealso \code{\link{get_IMIFA_results}}, \code{\link{mcmc_IMIFA}}
 #'
+#' @author Keefe Murphy
+#'
 #' @examples
 #' UUU <- PGMM_dfree(Q=4:5, P=50, G=3, method="UUU")
 #' CCC <- PGMM_dfree(Q=4:5, P=50, G=3, method="CCC")
@@ -418,6 +424,8 @@
 #' @export
 #' @seealso \code{\link{get_IMIFA_results}}, \code{\link[slam]{as.simple_triplet_matrix}}, \code{\link[stats]{hclust}}
 #'
+#' @author Keefe Murphy
+#'
 #' @examples
 #' # Run a IMIFA model and extract the sampled cluster labels
 #' # data(olive)
@@ -470,6 +478,36 @@
         return(list(rate2 = a.prob >= 0  || - rexp(1)  < a.prob, sw = sw))
     }
 
+  # Positive-(Semi)Definite Checker
+
+    #' Check Postive-(Semi)definiteness of a matrix
+    #'
+    #' Tests whether all eigenvalues of a symmetric matrix are positive (or strictly non-negative).
+    #' @param x A matrix, assumed to be real and symmetric.
+    #' @param tol Tolerance for singular values and for absolute eigenvalues - only those with values larger than tol are considered non-zero (default: tol = \code{max(dim(x))*max(E)*.Machine$double.eps}, where \code{E} is the vector of absolute eigenvalues).
+    #' @param semi Logical switch to test for positive-semidefiniteness when \code{TRUE} or positive-definiteness when \code{FALSE} (the default).
+    #'
+    #' @return A logical value (\code{TRUE} or \code{FALSE})
+    #' @export
+    #'
+    #' @examples
+    #' x <- cov(matrix(rnorm(100), nrow=10, ncol=10))
+    #' is.posi_def(x)
+    #' is.posi_def(x, semi=TRUE)
+    is.posi_def <- function(x, tol = NULL,   semi = FALSE)  {
+      if(!is.matrix(x)   &&
+         nrow(x) != ncol(x))               stop("argument x is not a square matrix")
+      if(!is.symmetric(x))                 stop("argument x is not a symmetric matrix")
+      if(!is.numeric(x))                   stop("argument x is not a numeric matrix")
+      eigs      <- eigen(x,  only.values = TRUE,  symmetric = TRUE)$values
+      abseigs   <- abs(eigs)
+      tol       <- if(missing(tol)) max(abseigs)  * nrow(x) * .Machine$double.eps else tol
+      if(length(tol) > 1 ||
+         !is.numeric(tol))                 stop("argument tol is not a single number")
+      test      <- replace(eigs, abseigs < tol, 0)
+        !any(if(isTRUE(semi)) test < 0 else test <= 0)
+    }
+
   # Length Checker
     .len_check  <- function(obj0g, switch0g, method, P, range.G, P.dim = TRUE) {
       V         <- ifelse(P.dim, P, 1L)
@@ -513,6 +551,8 @@
 #' @export
 #' @seealso \code{\link{G_variance}}, \code{\link{G_priorDensity}}, \code{\link[Rmpfr]{Rmpfr}}
 #'
+#' @author Keefe Murphy
+#'
 #' @examples
 #' G_expected(N=50, alpha=19.23356)
 #'
@@ -551,6 +591,8 @@
 #' @export
 #' @seealso \code{\link{G_expected}}, \code{\link{G_priorDensity}}, \code{\link[Rmpfr]{Rmpfr}}
 #'
+#' @author Keefe Murphy
+#'
 #' @examples
 #' G_variance(N=50, alpha=19.23356)
 #'
@@ -586,17 +628,6 @@
       }
     })
 
-  # Detach packages
-    .detach_pkg <- function(pkg, character.only = FALSE) {
-      if(!character.only) {
-        pkg     <- deparse(substitute(pkg))
-      }
-      searches  <- paste("package", pkg, sep=":")
-      while(searches %in% search()) {
-        detach(searches, unload=TRUE, character.only=TRUE)
-      }
-    }
-
   # Print functions
 #' @method print IMIFA
 #' @export
@@ -687,14 +718,32 @@
         cat(paste0(capture.output(print.Results_IMIFA(object)), msg))
     }
 
-    .power2     <- function(x) x * x
-    .which0     <- function(x) which(x == 0)
-    .chol       <- function(x) tryCatch(chol(x), error=function(e) chol(make.positive.definite(x)))
+  # Other Hidden Functions
+    .chol       <- function(x) tryCatch(chol(x), error=function(e) chol(.make_posdef(x)))
+
+    .detach_pkg <- function(pkg, character.only = FALSE) {
+      if(!character.only) {
+        pkg     <- deparse(substitute(pkg))
+      }
+      searches  <- paste("package", pkg, sep=":")
+      while(searches %in% search()) {
+        detach(searches, unload=TRUE, character.only=TRUE)
+      }
+    }
+
+    .ent_exit  <- function(opts = options()) {
+      ent      <- readline("Hit <Return> to see next plot or type 'EXIT'/hit <Esc> to exit: ")
+      options(show.error.messages=FALSE)
+      on.exit(suppressWarnings(options(opts)), add=TRUE)
+        if(ent  %in% c("exit", "EXIT"))    stop()
+    }
+
     .ledermann  <- function(N, P) {
       R         <- P + 0.5 - (0.5  * sqrt(8 * P + 1))
         as.integer(floor(min(N - 1, ifelse(1e-10 > abs(R - round(R)), round(R), R))))
     }
-      .logitdensity <- function(x)  {
+
+    .logitdensity   <- function(x)  {
       y         <- qlogis(x)
       g         <- density(y, bw    = "SJ")
       xgrid     <- plogis(g$x)
@@ -702,4 +751,53 @@
       g$x       <- xgrid
         return(g)
     }
+
+   .make_posdef <- function(x, tol = NULL) {
+      if(!is.matrix(x)   &&
+        nrow(x) != ncol(x))                stop("argument x is not a square matrix")
+      if(!is.symmetric(x))                 stop("argument x is not a symmetric matrix")
+      if(!is.numeric(x))                   stop("argument x is not a numeric matrix")
+      d         <- nrow(x)
+      eigs      <- eigen(x, symmetric = TRUE)
+      eval      <- eigs$values
+      evec      <- eigs$vectors
+      tol       <- if(missing(tol)) max(abs(eval)) * d * .Machine$double.eps else tol
+        return(x + evec %*% tcrossprod(diag(pmax(0,  2 * tol - eval), d), evec))
+    }
+
+    .moore_inv  <- function(X, tol = sqrt(.Machine$double.eps)) {
+      if(length(dim(X)) > 2L ||
+         !(is.numeric(X)     ||
+           is.complex(X)))                 stop("'X' must be a numeric or complex matrix")
+      if(!is.matrix(X))    X <- as.matrix(X)
+      Xs        <- svd(X)
+      if(is.complex(X)) Xs$u <- Conj(Xs$u)
+      Posi      <- Xs$d > max(tol * Xs$d[1L], 0)
+      if(all(Posi))     {
+          Xs$v %*% (t(Xs$u)/Xs$d)
+      } else if(!any(Posi))   {
+          array(0, dim(X)[2L:1L])
+      } else {
+          Xs$v[,Posi,drop=FALSE] %*% (t(Xs$u[,Posi,drop=FALSE])/Xs$d[Posi])
+      }
+    }
+
+    .power2     <- function(x) x * x
+
+    .Procrustes <- function(X, Xstar, translate = FALSE, dilate = FALSE) {
+      n         <- nrow(X)
+      p         <- ncol(X)
+      if(n      != nrow(Xstar))            stop("X and Xstar do not have the same number of rows")
+      if(p      != ncol(Xstar))            stop("X and Xstar do not have the same number of columns")
+      J         <- if(translate) diag(n) - matrix(1/n, n, n)                         else diag(n)
+      C         <- crossprod(Xstar, J) %*% X
+      svdX      <- svd(C)
+      R         <- tcrossprod(svdX$v, svdX$u)
+      s         <- if(dilate)    sum(diag(C %*% R))/sum(diag(crossprod(X, J) %*% X)) else 1
+      tt        <- if(translate) crossprod(Xstar - s * X %*% R, matrix(1, n, 1))/n   else 0
+      Xnew      <- s * X %*% R + if(translate) matrix(tt, n, p, byrow = TRUE)        else tt
+        return(c(list(X.new = Xnew), list(R = R), if(translate) list(tt = tt), if(dilate) list(s = s)))
+    }
+
+    .which0     <- function(x) which(x == 0)
     #
